(* :Title: Options *)

(* :Context: QuantLib`Options` *)

(* :Author: Niels Elken Snderby *)

(* :Mathematica Version: 4.0 *)

(* :Copyright: Copyright (C) 2003 Niels Elken Snderby *)

(* :License:

   This file is part of QuantLib for Mathematica, a Mathematica extension for
   QuantLib, a free-software/open-source financial C++ library
   - http://www.nielses.dk/quantlib/mma/
   - http://quantlib.org/

   QuantLib for Mathematica is free software: you can redistribute it and/or 
   modify it under the terms of the QuantLib license.  You should have 
   received a copy of the license along with this program; if not, please 
   email ferdinando@ametrano.net The license is also available online at 
   http://quantlib.org/html/license.html

   This program is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
   or FITNESS FOR A PARTICULAR PURPOSE. See the license for more details.
*)

BeginPackage["QuantLib`Options`",
"QuantLib`Common`QuantLibCommon`" (* Needs SymbolToNumber functionality *)
]

(* Usage messages *)

Keyfigures::usage = 
"Keyfigures[instrument, model] gives all keyfigures (including Value)
for the given instrument under the given model."

Value::usage = 
"Value[instrument, model] is a short form 
for Value /. Keyfigures[instrument, model] 
and gives the value of the instrument under the given model."

CallOption::usage =
"CallOption is an option type where the buyer has the right to buy the 
underlying at maturity."

PutOption::usage =
"PutOption is an option type where the buyer has the right to sell the 
underlying at maturity."

EuropeanOption::usage = 
"EuropeanOption[strike, residualtime, type] represents a european
option of the given type (CallOption or PutOption), exercise price
(strike) and time to maturity (residualtime)."

BlackScholesModel::usage =
"BlackScholesModel[stockprice, riskfreerate, dividend, vol] represents a
model where the underlying asset is worth stockprice at time zero and the
risk free rate, dividend yield and volatility are as given."

Begin["`Private`"]

(* Give access to QuantLib`Private` context as LinkPatterns for external 
   program are there. Does a more elegant solution exist? *)
AppendTo[$ContextPath, "QuantLib`Private`"] 

(* Translation table for passing "enum"-values to C++ *)
OptionTypeNumbers = { 
{ CallOption, 0 },
{ PutOption, 1 }
}

Keyfigures[EuropeanOption[strike_, maturity_, type_],
	   BlackScholesModel[underlying_, riskFreeRate_, 
	                     dividendYield_, volatility_]] ^:= 
	   qlEuropeanOption[SymbolToNumber[type, OptionTypeNumbers],
	                        underlying, strike, 
	                        dividendYield, riskFreeRate,
                                maturity, volatility]

Value[x___] := Value /. Keyfigures[x]

End[]

EndPackage[]
