/*
 Copyright (C) 2003 Niels Elken Snderby

 This file is part of QuantLib for Mathematica, a Mathematica extension for
 QuantLib, a free-software/open-source financial C++ library
 http://www.nielses.dk/quantlib/mma
 http://quantlib.org/

 QuantLib for Mathematica is free software: you can redistribute it and/or
 modify it under the terms of the QuantLib license.  You should have received
 a copy of the license along with this program; if not, please email
 ferdinando@ametrano.net The license is also available online at
 http://quantlib.org/html/license.html

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the license for more details.
*/

#include "mmamontecarlo.hpp"

void qlMonteCarloPathBlackScholes(double s0, double riskFreeRate,
                                  double dividendYield, double volatility,
                                  double length, int timeSteps)
{
    try {
        using namespace QuantLib::MonteCarlo;

        // generate the risk neutral path
        double mu = riskFreeRate - dividendYield
                                 - 0.5 * volatility * volatility;

        static long seed = 0;

        if (seed == 0)
            seed = QL_TIME(0);
        else
            seed++;

        GaussianPathGenerator
            pg(mu, volatility*volatility, length, timeSteps, seed);

        const Sample<Path>& sample = pg.next();
        Path path = sample.value;

        double *ret = new double[timeSteps];

        double s = s0;

        for (Size i = 0; i < timeSteps; i++) {
            s += s * path[i];
            ret[i] = s;
        }

        MLPutRealList(stdlink, ret, timeSteps);
        delete[] ret;
    } QLML_HANDLE_EXCEPTIONS
}
