/*
 Copyright (C) 2003 Niels Elken Snderby

 This file is part of QuantLib for Mathematica, a Mathematica extension for
 QuantLib, a free-software/open-source financial C++ library
 http://www.nielses.dk/quantlib/mma
 http://quantlib.org/

 QuantLib for Mathematica is free software: you can redistribute it and/or
 modify it under the terms of the QuantLib license.  You should have received
 a copy of the license along with this program; if not, please email
 ferdinando@ametrano.net The license is also available online at
 http://quantlib.org/html/license.html

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the license for more details.
*/

#include "mmaoptions.hpp"

void qlEuropeanOption(int type, double underlying, double strike,
                        double dividendYield, double riskFreeRate,
                        double maturity, double volatility)
{
    try {
        FlatPlainOption option((Option::Type)type, underlying, strike, dividendYield,
                               riskFreeRate, maturity, volatility);

        // Make all calculations first, so that if an error is thrown,
        // it can safely be send to Mathematica. (More elegant solution exists?)
        double NPV = option.NPV();
        double delta = option.delta();
        double gamma = option.gamma();
        double theta = option.theta();
        double vega = option.vega();
        double rho = option.rho();
        double dividendRho = option.dividendRho();

        MLPutFunction(stdlink, "List", 7);
        QLMLPutRule(stdlink, "Value", NPV);
        QLMLPutRule(stdlink, "Delta", delta);
        QLMLPutRule(stdlink, "Gamma", gamma);
        QLMLPutRule(stdlink, "Theta", theta);
        QLMLPutRule(stdlink, "Vega", vega);
        QLMLPutRule(stdlink, "Rho", rho);
        QLMLPutRule(stdlink, "DividendRho", dividendRho);
    } QLML_HANDLE_EXCEPTIONS
}
